#!/bin/bash
# Copyright 2008-2011 Matus Chochlik. Distributed under the Boost
# Software License, Version 1.0. (See accompanying file
# LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
#
install_prefix=
function set_inst_prefix()
{
	if [ "$1" == "" ]
	then echo "Invalid prefix ''. Did you mean '/' ?" && return 1
	fi
	if [ "$install_prefix" != "" ]
	then echo "Multiple prefixes provided!" && return 2
	fi
	install_prefix="$1"
	return 0	
}
#
function print_usage()
{
	echo "Usage bootstrap [options]"
	echo "  options:"
	echo "    -h --help: Print this usage screen"
	echo "    --prefix=<path>:  Installation prefix"
	echo
	return 0
}
# if we have getopt check the command line options
if [ -x "$(which getopt 2> /dev/null)" ]
then
	parsed_options=$(getopt \
		--name=bootstrap \
		--options=h \
		--longoptions=help,prefix: \
		-- "$@" \
	)
	[[ $? -eq 0 ]] || exit $?
	eval set -- ${parsed_options}
else echo "Warning: getopt not found."
fi
#
while [ $# -gt 0 ]
do
	case ${1} in
		-h|--help) print_usage && exit $?;;
		--prefix=*) set_inst_prefix ${1#*=} || exit $?;;
		--prefix) shift && set_inst_prefix ${1} || exit $?;;
		--) shift && break;;
		*)  echo "Invalid argument '$1'" && exit 10;;
	esac
	shift
done
#
#
cmake $(dirname $0) \
	${install_prefix:+-DCMAKE_INSTALL_PREFIX=}${install_prefix}
