.SH "INPUT PATTERNS"

Input patterns specify which C++ input source or header files
and which constructs from these files should be processed into
the resulting registering code.
Every input pattern must have a matching output pattern (see below).
The input patters can be specified on the command line
when invoking \fImaureen\fR, or in one of the \fBMAREfiles\fR
specified with the \fB-m\fR command line options.
An input pattern has the following format:

.br
	'root|path|file|suffix|kind|scope|name'
.br

where the \fB|\fR (pipe) symbol acts as the separator and
the individual components have the following meaning:

.B root
The root path to the sources to be processed.
The special value \fI%\fR means that the input files
will be searched at the paths specified by the \fB-I\fR options.
.br
There is an important concept related to the root component of
the pattern: the \fBbase directory\fR. In case the input pattern
was specified on the command line then the base directory is
the current working directory from which \fImaureen\fR has been
invoked. If the pattern was specified in a \fBMAREfile\fR, then
the base directory is the directory where the \fBMAREfile\fR
is located.
An empty value specifies that the \fBbase directory\fR is to be
used as root.
.B Examples:
/usr/include, /home/login/include, C:\\Devel\\MyProject\\headers, %.

.B path
The path to the sources relative to the root. The Special value
\fI%\fR means that any path applies including an empty path.
This way any file in the root directory or its subdirectories
can be specified.
.B Examples:
src, test, %.

.B file
The file name pattern without the suffix, The special value \fB%\fR
acts as a wildcard for a string of arbitrary length.
.B Examples:
foo, foo%, %.

.B suffix
The file suffix (extension) pattern. The special value \fB%\fR
acts as a wildcard for a string of arbitrary length.
.B Examples:
h, hpp, h%, %.

.B kind
The kind of programming construct to be processed. The special value
\fB%\fR can be used to process everything.
.B Examples:
namespace, class, enum, %.

.B scope
The pattern for the scopes to be processed. The special value \fB%\fR
represents any scope, an empty value represents the global scope only.
.B Examples:
std, boost, foo%, %.

.B name
The pattern specifying the names of the language constructs
(specified by the \fBkind\fR component) to be processed.
The special value \fB%\fR can be used to process everything.
.B Examples:
tm, my_class, my_project, %.

.B Examples or full input patterns:
.nr ie 0 1

.br
\n+(ie: '%|%|%|%|%|%|%'
.br

Process files of any type in the directories specified by the \fB-I\fR options and
their subdirectories, register everything (namespaces, classes, enums, etc.),
in the global scope or any other namespace.

.br
\n+(ie: '%|%|%|hpp|%|%|%'
.br

Process \fBhpp\fR files in the directories specified by the \fB-I\fR options and
their subdirectories, register everything (namespaces, classes, enums, etc.),
in the global scope or any other namespace.

.br
\n+(ie: '|%|%|hpp|%|%|%'
.br

Process \fBhpp\fR files in the base directory and
its subdirectories, register everything,
in the global scope or any other namespace.

.br
\n+(ie: '||%|hpp|%|%|%'
.br

Process \fBhpp\fR files in the base directory but not
its subdirectories, register everything,
in the global scope or any other namespace.

.br
\n+(ie: '||%|hpp|namespace|%|%'
.br

Process \fBhpp\fR files in the base directory but not
its subdirectories, register only \fBnamespace\fRs, both in the
global scope and nested ones.

.br
\n+(ie: '||%|hpp|namespace||%'
.br

Process \fBhpp\fR files in the base directory but not
its subdirectories, register only \fBnamespace\fRs form the global
scope.

.br
\n+(ie: '|%|%|inc|%||%'
.br

Process \fBinc\fR files in the base directory and
its subdirectories, register everything,
from the global scope, but not from any other namespace.

.br
\n+(ie: '|%|%|hpp|%|test|%'
.br

Process \fBhpp\fR files in the base directory and
its subdirectories, register everything,
from the \fBtest\fR namespace, but not its nested namespaces.

.br
\n+(ie: '|%|%|hpp|%|foo%|%'
.br

Process \fBhpp\fR files in the base directory and
its subdirectories, register everything,
from the \fBfoo\fR namespace and its nested namespaces.

.br
\n+(ie: '|%|%|hpp|class|%|foo'
.br

Process \fBhpp\fR files in the base directory and
its subdirectories, register only classes named \fBfoo\fR 
from the global scope or any other namespace.

.br
\n+(ie: '%||time|h|%|std|tm'
.br

Process the \fBtime.h\fR file in the input search directories
but not in their subdirectories, register anything
named \fBtm\fR from the \fBstd\fR namespace.

.br
\n+(ie: '%||time||struct|std|tm'
.br

Process the \fBtime\fR file in the input search directories
but not in their subdirectories, register only \fBstruct\fRs
named \fBtm\fR from the \fBstd\fR namespace.

.SH "OUTPUT PATTERNS"

Output patterns specify the names and locations of the output
files containing the resulting Mirror's registering code. Every output
pattern is linked to an input pattern 
The output patters can be specified on the command line
when invoking \fImaureen\fR, or in one of the \fBMAREfiles\fR
specified with the \fB-m\fR command line options.
.br
As with the input pattern the output pattern has several
components which can contain literal strings (names, paths),
but there are also several variables which can reference
the values of the components from the related input pattern.
The variables are geneally expanded by the $(\fIvariable-name\fR)
expression although several of the variable-expansion expressions
can also have an extended syntax:

.B root
References the root from the input pattern

.B path
References the path of the input file

.B file
References the basename of the input file

.B suffix
References the suffix of the input file

.B kind
References the kind of the processed (registered) language
construct (namespace, class, enum, etc.)

.B scope
References the scope of the processed (registered) language
construct. The separating \fB::\fR (double-colon) can be
replaced in the expansion by another separator with this
expression syntax $(\fIscope:\fB<separating character>\fR),
for example for a type \fBfoo::bar::baz::foobar\fR the
expression $(\fIscope:\fB/\fR) expands into \fB'foo/bar/baz'\fR
and $(\fIscope:\fB-\fR) expands into \fB'foo-bar-baz'\fR.

.B name
References the name of the processed (registered) language
construct.

An output pattern has the following format:

.br
	'root|path|file|suffix'
.br

where the \fB|\fR (pipe) symbol acts as the separator and
the individual components have the following meaning:

.B root
The directory path where the output files should be placed.
As with the input pattern there is the concept of the
\fBbase directory\fR: In case the output pattern
was specified on the command line then the base directory is
the current working directory from which \fImaureen\fR has been
invoked. If the pattern was specified in a \fBMAREfile\fR, then
the base directory is the directory where the \fBMAREfile\fR
is located.
The path specified as root is relative to the \fBbase directory\fR,
unless the value is an absolute path.
.B Examples:
/opt/my_project/meta-data, ./meta-data, ../output/meta-data, $(root).

.B path
The path to the output files relative to the root.
If an empty value is specified, then the output files are
placed directly into the root directory.
.B Examples:
foo, test, $(path).

.B name
The basename of the output file.
.B Examples:
$(name), $(scope:-)-$(name), meta-$(name), meta-data.

.B suffix
The suffix of the output file.
.B Examples:
hpp, metahpp, meta$(kind).

.B Examples or full output patterns:
.nr oe 0 1

.br
\n+(oe: '../output/meta-data|$(path)|$(file)|$(suffix)'
.br

Place the output files to the \fB../output/meta-data\fR directory
at the same paths that the input files had, relative to the root
specified in the input pattern.

.br
\n+(oe: '../output/meta-data|$(scope:/)|$(name)|hpp'
.br

Place the output files to the \fB../output/meta-data\fR directory,
into subdirectories for every namespace, recursively for nested
namespaces, the files named by the individual registered
language constructs, with the \fBhpp\fR extension.

.br
\n+(oe: 'meta-data||$(scope:-)-$(name)|hpp'
.br

Place the output files directly into the \fBmeta-data\fR
directory, the files named by the full-nested-names of the individual
registered language constructs (the double-colons replaced by \fB-\fR),
with the \fBhpp\fR extension.

.br
\n+(oe: '|$(scope:_)|$(name)|meta-$(kind)'
.br

Place the output to the base directory, into subdirectories
for every namespace (the double-colons in the nested namespace
named replaced by an underscore), the files named by the basic
names of the individual registered constructs, with extensions
like \fBmeta-struct\fR for struct's, \fBmeta-class\fR for classes,
etc.


.SH "MAREFILES"

Besides specifying the input-to-output processing options
on the command line, they can be also be specified inside
special files, called \fBMAREfile\fRs (from Mirror Auto
Reflection Engine files).
When invoking \fImaureen\fR, one or multiple MAREfiles
can be specified explicitly with the \fB-m\fR option,
or if there is a file with the name \fBMAREfile\fR
in the current working directory from which \fImaureen\fR
was invoked, then it is used implicitly (unless there
are any marefiles specified explicitly; in such case
the \fBMAREfile\fR is not used).

The marefiles can contain three types of lines;
comments, commands and input/output pattern pairs.

Comment lines must begin with the \fB#\fR sign and are
ignored by the processor. Comments cannot be used on
the same line with commands or input/output pattern pairs.

.B Examples:
.br
# This is a comment
.br
 # ERROR - invalid comment
.br
@ 'define|TEST' # ERROR - invalid comment

Command lines must begin with the \fB@\fR (at) symbol
and have the following format (note that the command
with the potential parameters is enclosed in apostrophes):

	@ '\fI<command>\fR[|\fIparameter1\fR[|\fIparameter2\fR[|...]]]'

.B Examples:
.br
	@ 'define|_DEBUG'
.br
	@ 'define|__BEGIN_STD_NAMESPACE|namespace std {'
.br
	@ 'define|__END_STD_NAMESPACE|}'
.br

The input/output pattern pair simply contains an input and an
output pattern, separated by a space on a single line (again,
note that both patterns are enclosed in apostrophes):

.br
	'\fI<input-pattern>\fR' '\fI<output-pattern>\fR'
.br

The format of the patterns is the same as specified in the
\fBINPUT PATTERNS\fR and \fBOUTPUT PATTERNS\fR sections above.

.B Examples:
.br
	'|%|%|hpp|namespace||%' '../out/meta-data||$(name)|$(suffix)'
.br
	'|%|%|hpp|%|%|%' '../out/meta-data||$(scope:_)|$(suffix)'
.br

.SH "EXAMPLES"

This section shows several simple examples including the sources,
MAREfiles and \fImaureen\fR command-line invocations.

Let's suppose there is the following directory tree:

.sp
.in +4n
.nf
\fC
+-+-[root_dir/]
  |
  +-+-[include/]
  | |
  | +---[fubar.hpp]
  | +---[person.hpp]
  | `---[tetrahedron.hpp]
  |
  +-+-[maureen/]
  | |
  | +---[MAREfile.1]
  | +---[MAREfile.2]
  | `---[MAREfile.3]
  |
  `---[MAREfile]
\fR
.fi
.in

where the header files contain the following definitions:
.sp
.in +4n
.nf
\fC
// --- fubar.hpp ---

namespace foo {
namespace bar {
namespace baz {

struct qux { };

} // namespace baz
} // namespace bar
} // namespace foo

// --- person.hpp ---

namespace test {

struct person_pod
{
	std::string given_name;
	std::string middle_name;
	std::string family_name;
	std::tm birth_date;
	double weight;
	double height;
};

} // namespace test

// --- tetrahedron.hpp ---

namespace test {

struct vector
{
	double x,y,z;

	vector(double _x, double _y, double _z);
	vector(double _w);
	vector(void);

	double length(void) const;
	friend vector operator + (const vector& a, const vector& b);
	friend vector operator - (const vector& a, const vector& b);
	// cross produnt
	friend vector operator % (const vector& a, const vector& b);
	// dot product
	friend double operator * (const vector& a, const vector& b);
};

struct triangle
{
	vector a, b, c;

	triangle(const vector& _a, const vector& _b, const vector& _c);
	triangle(void);

	double area(void) const;
};

struct tetrahedron
{
	triangle base;
	vector apex;

	tetrahedron(const triangle& _base, const vector& _apex);

	tetrahedron(
		const vector& a,
		const vector& b,
		const vector& c,
		const vector& d
	);

	const vector& a(void);
	const vector& b(void);
	const vector& c(void);
	const vector& d(void);

	void reset_apex(const vector& _apex);
};

} // namespace test
\fR
.fi
.in

The MAREfiles have the following contents:
.sp
.in +3n
.nf
\fC
 # Marefile
 # Process everything in the global scope
 '|%|%|hpp|%||%' 'out/metadata||$(name)|$(suffix)'
 # Process everything in the nested namespaces
 '|%|%|hpp|%|%|%' 'out/metadata|$(scope:/)|$(name)|$(suffix)'

 # maureen/MAREfile.1
 # Process everything in the global scope
 '|%|%|hpp|%||%' '../out/metadata||$(name)|$(suffix)'
 # Process everything in the nested namespaces
 '|%|%|hpp|%|%|%' '../out/metadata|$(scope:/)|$(name)|$(suffix)'

 # maureen/MAREfile.2
 # process namespaces in the global scope
 '|%|%|hpp|namespace||%' '../out/metadata||$(name)|$(suffix)'
 # process everything in the namespaces
 '|%|%|hpp|%|%|%' '../out/metadata||$(scope:_)|$(suffix)'
\fR 
.fi
.in

Invoking \fImaureen\fR from the \fBroot_dir\fR without any arguments,
which causes it to use the \fBMAREfile\fR in the root dir, produces
the following directory tree:

.sp
.in +4n
.nf
\fC
+-+-[root_dir/]
  |
  +-+-[out/]
  | |
  | `-+-[metadata/]
  |   |
  |   +-+-[foo/]
  |   | |
  |   | +-+-[bar/]
  |   | | |
  |   | | +-+-[baz/]
  |   | | | |
  |   | | | `---[qux.hpp]
  |   | | |
  |   | | `---[baz.hpp]
  |   | |
  |   | `---[bar.hpp]
  |   |
  |   +-+-[test/]
  |   | |
  |   | +---[person_pod.hpp]
  |   | +---[tetrahedron.hpp]
  |   | +---[triangle.hpp]
  |   | `---[vector.hpp]
  |   |
  |   +---[foo.hpp]
  |   `---[test.hpp]
  |
  `---[...]
\fR
.fi
.in

The directory \fBout/metadata\fR is the output root. The files
\fBfoo.hpp\fR and \fBtest.hpp\fR contain code registering the
namespace \fBfoo\fR and \fBtest\fR respectivelly, with Mirror.
The directory \fBtest/\fR contains registering code for types
defined in the \fBtest\fR namespace. The files inside are named
by the individual classes declared therein and contain
registering code for the respective classes.

The directory \fBfoo\fR and its nested subdirectories \fBbar\fR
and \fBbaz\fR contain headers for registering the constructs
defined in the namespaces \fBfoo\fR, \fBfoo::bar\fR and 
\fBfoo::bar::baz\fR, respectivelly. The file \fBqux.hpp\fR for example
contains registering code for \fBstruct foo::bar::baz::qux\fR,
declared in the \fBroot_dir/include/fubar.hpp\fR input header
file.

The same output as above, could be achieved by the following
invocations of \fImaureen\fR:

.sp
.in +4n
.nf
\fC
user@host:.../\fBroot_dir\fR$ \fImaureen\fR \\
.br
    --input '|%|%|hpp|%||%'  --output 'out/metadata||$(name)|$(suffix)' \\
.br
    --input '|%|%|hpp|%|%|%' --output 'out/metadata|$(scope:/)|$(name)|$(suffix)'
.br

user@host:.../\fBroot_dir\fR$ \fImaureen\fR --marefile \fBmaureen/MAREfile.1\fR
\fR
.br
.fi
.in

Invoking \fImaureen\fR with the \fB-m maureen/MAREfile.2\fR option
produces the following output tree:

.sp
.in +4n
.nf
\fC
+-+-[root_dir/]
  |
  +-+-[out/]
  | |
  | `-+-[metadata/]
  |   |
  |   +---[foo.hpp]
  |   +---[foo_bar.hpp]
  |   +---[foo_bar_baz.hpp]
  |   `---[test.hpp]
  |
  `---[...]
\fR
.fi
.in

Again, the directory \fBout/metadata\fR is the output root.
The files \fBfoo.hpp\fR,  \fBfoo_bar.hpp\fR, \fBfoo_bar_baz.hpp\fR
and \fBtest.hpp\fR contain code registering everything declared
in the \fBfoo\fR, \fBfoo::bar\fR, \fBfoo::bar::baz\fR and \fBtest\fR
namespaces, respectivelly, with Mirror.

.SH "AUTHOR"
Matus Chochlik, Matus.Chochlik@fri.uniza.sk


.SH "COPYRIGHT"
Copyright (c) 2008, 2009, 2010, 2011 Matus Chochlik
.PP
Permission is granted to copy, distribute and/or modify this document
under the terms of the Boost Software License, Version 1.0. 
(See a copy at http://www.boost.org/LICENSE_1_0.txt)

