#!/bin/bash
# Copyright 2009-2011 Matus Chochlik. Distributed under the Boost
# Software License, Version 1.0. (See accompanying file
# LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
#
# check the prerequisities
[[ -x "$(which doxygen)" ]] || exit 101
[[ -x "$(which xsltproc)" ]] || exit 102
#
common_dir="$(dirname $0)/common"
module_dir="$(dirname $0)/$(basename $0 .sh)"
# the working directory
work_dir="$(mktemp -d)"
# cleanup the temporary working directory on exit
trap "rm -rf ${work_dir}" EXIT
#
# include the common helper scripts
source ${common_dir}/normalize_filepath
#
# setup the temporary doxyfile
doxyfile=${work_dir}/Doxyfile
doxyfile_input=${work_dir}/Doxyfile.input
doxyfile_inc_path=${work_dir}/Doxyfile.inc_path
include_search_paths=${work_dir}/doxyreg.inc_paths
# copy the common things
echo "@INCLUDE = ${module_dir}/Doxyfile.common" > ${doxyfile}
# set the output directory
echo "OUTPUT_DIRECTORY = \"${work_dir}\"" >> ${doxyfile}
#
#
# options
defines=""
include_paths=""
# process the options
while read option_mark option
do
	[[ "${option_mark}" == "@" ]] || exit 1
	[[ "${option}" == "-" ]] && break
	eval set ${option//|/\' \'}
	case "${1}" in
		define) defines="${defines} \\\\\n\t\"${2}${3:+=}${3}\"";;
		macro_prefix) macro_prefix="${2}";;
		include) echo ${2} >> ${include_search_paths};;
		*);;
	esac
done
#
echo -e "PREDEFINED = ${defines}" >> ${doxyfile}
echo -n "INPUT = " > ${doxyfile_input}
echo -n "INCLUDE_PATH = " > ${doxyfile_inc_path}
#
# setup the configuration xml file
config_xml=${work_dir}/config.xml
echo "<?xml version='1.0' encoding='utf-8'?>" > ${config_xml}
echo "<doxyreg>" >> ${config_xml}
# process the input and output patterns
while read input_pattern output_pattern
do
	# use the common helper to parse the input pattern
	source $(dirname $0)/common/parse_input_pattern
	# find all files matching the input pattern
	if [ "${input_root}" != "%" ]
	then echo "${input_root}"
	else cat ${include_search_paths};
	fi |
	while read input_root
	do
		source $(dirname $0)/common/input_filepath_regex
		find "${input_root}" -regex "${input_filepath_regex}" |
		while read tmp_input_filepath
		do
			input_filepath=${tmp_input_filepath#./}
			# parse the output pattern
			source $(dirname $0)/common/parse_output_pattern
			#
			# add the input file to the doxyfile input lise
			echo " \\" >> ${doxyfile_input}
			echo -n " 	\"${input_filepath}\"" >> ${doxyfile_input}
			#
			# add a include directory
			echo " \\" >> ${doxyfile_inc_path}
			echo -n " 	\"${input_root}\"" >> ${doxyfile_inc_path}
			#
			# update the config xml file
			echo "<input" >> ${config_xml}
			if [ "${input_filepath:0:1}" == "/" ]
			then echo "	full_path='${input_filepath}'" >> ${config_xml}
			else
				echo -n "	full_path='" >> ${config_xml}
				echo -n "${PWD}/${input_filepath}" |
				sed "${normalize_filepath_sed}" >> ${config_xml}
				echo "'" >> ${config_xml}
			fi
			echo "	root='${input_real_root}'" >> ${config_xml}
			echo "	path='${input_real_path}'" >> ${config_xml}
			echo "	file='${input_real_file}'" >> ${config_xml}
			echo "	suffix='${input_real_suffix}'" >> ${config_xml}
			echo "	kind='${input_kind}'" >> ${config_xml}
			echo "	scope='${input_scope}'" >> ${config_xml}
			echo "	name='${input_name}'" >> ${config_xml}
			echo ">" >> ${config_xml}
			echo "<output" >> ${config_xml}
			echo "	root='${output_root}'" >> ${config_xml}
			echo "	path='${output_path}'" >> ${config_xml}
			echo "	file='${output_file}'" >> ${config_xml}
			echo "	suffix='${output_suffix}'" >> ${config_xml}
			echo "/>" >> ${config_xml}
			echo "</input>" >> ${config_xml}
		done
	done
done
# put a newline to the doxyfile
echo >> ${doxyfile}
echo "@INCLUDE = ${doxyfile_input}" >> ${doxyfile}
echo "@INCLUDE = ${doxyfile_inc_path}" >> ${doxyfile}
#
# finalize the configuration xml
echo "</doxyreg>" >> ${config_xml}
#
# process the input files with doxygen
doxygen ${doxyfile}
# transform the output into the intermediate file
xsltproc \
	--stringparam work_dir "${work_dir}" \
	--stringparam macro_prefix "${macro_prefix}" \
	${module_dir}/xslt/doxyreg_process.xsl \
	${work_dir}/xml/index.xml |
xsltproc \
	--stringparam run_path "${PWD}" \
	--stringparam macro_prefix "${macro_prefix}" \
	${module_dir}/xslt/doxyreg_split.xsl \
	- |
while read input_line
do
	case "${input_line:0:2}" in
		@@) output_file="${input_line:2}"; (
			mkdir -p $(dirname "${output_file}") &&
			rm -f "${output_file}" &&
			touch "${output_file}"
		  ) || exit 1;;
		@*) output_file="${input_line:1}";;
		*)  echo "${input_line}" >> "${output_file}" || exit 2;;
	esac
done
