<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:crypto="http://exslt.org/crypto"
  extension-element-prefixes="crypto"
  version="1.0"
>
<!--
  - Copyright 2008-2010 Matus Chochlik. Distributed under the Boost
  - Software License, Version 1.0. (See accompanying file
  - LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
  -->

  <xsl:output method="text"/>
  <xsl:strip-space elements="*"/>
  <xsl:preserve-space elements="xsl:text"/>
  <xsl:param name="run_path"/>
  <xsl:param name="macro_prefix"/>

  <xsl:include href="newline.xsl"/>

  <xsl:template match="*">
  </xsl:template>

  <xsl:template match="/items">
    <xsl:for-each select="item">
      <xsl:variable name="output_rel_path" select="@output_rel_path"/>
      <xsl:variable name="output_path" select="concat($run_path, '/', $output_rel_path)"/>
      <!-- if this is the first part of the output file add a @ -->
      <xsl:if test="not(preceding-sibling::item[@output_rel_path = $output_rel_path])">@</xsl:if>
      <!-- append a tag saying where the output should go -->
      <xsl:text>@</xsl:text><xsl:value-of select="$output_path"/>
      <xsl:call-template name="newline"/>
      <!-- if this is the first part of the output file -->
      <xsl:if test="not(preceding-sibling::item[@output_rel_path = $output_rel_path])">
        <xsl:variable name="include_guard">
          <xsl:text>__MAUREEN_</xsl:text>
          <xsl:value-of select="translate(
              $output_rel_path,
              'abcdefghijklmnopqrstuvwxyz/.-:\\',
              'ABCDEFGHIJKLMNOPQRSTUVWXYZ______')
          "/>
          <xsl:text>_</xsl:text>
          <xsl:choose>
            <xsl:when test="function-available('crypto:md5')">
              <xsl:value-of select="crypto:md5(
                /items/item[@output_rel_path = $output_rel_path]/text()
              )"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:value-of select="generate-id(
                /items/item[@output_rel_path = $output_rel_path]/text()
              )"/>
            </xsl:otherwise>
          </xsl:choose>
          <xsl:text>_HPP</xsl:text>
        </xsl:variable>
        <!-- add the include guard -->
        <xsl:text>#ifndef </xsl:text><xsl:value-of select="$include_guard"/>
        <xsl:call-template name="newline"/>
        <xsl:text>#define </xsl:text><xsl:value-of select="$include_guard"/>
        <xsl:call-template name="newline"/>
        <!-- add the include directive for the base-level objects to be registered -->
        <xsl:call-template name="newline"/>
        <xsl:text>#include "</xsl:text>
        <xsl:value-of select="@input_include"/>
        <xsl:text>"</xsl:text>
        <xsl:call-template name="newline"/>
        <xsl:call-template name="newline"/>
        <!-- add the REG_BEGIN macro -->
        <xsl:value-of select="$macro_prefix"/><xsl:text>MIRROR_REG_BEGIN</xsl:text>
        <xsl:call-template name="newline"/>
      </xsl:if>
      <!-- print the registering code -->
      <xsl:value-of select="text()"/>
      <!-- if this is the last part of the output file -->
      <xsl:if test="not(following-sibling::item[@output_rel_path = $output_rel_path])">
        <!-- add the REG_END macro -->
        <xsl:call-template name="newline"/>
        <xsl:value-of select="$macro_prefix"/><xsl:text>MIRROR_REG_END</xsl:text>
        <xsl:call-template name="newline"/>
        <xsl:call-template name="newline"/>
        <xsl:text>#endif // include guard</xsl:text>
        <xsl:call-template name="newline"/>
      </xsl:if>
    </xsl:for-each>
  </xsl:template>

</xsl:stylesheet>
