<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
<!--
  - Copyright 2008-2010 Matus Chochlik. Distributed under the Boost
  - Software License, Version 1.0. (See accompanying file
  - LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
  -->

  <xsl:template match="ref[parent::type]" mode="processing_elaborated_type">
    <xsl:value-of select="document(concat($work_dir,'/xml/',@refid,'.xml'))/doxygen/compounddef/compoundname"/>
  </xsl:template>

  <xsl:template match="type" mode="processing_elaborated_type">
    <xsl:apply-templates mode="processing_elaborated_type"/>
  </xsl:template>

  <xsl:template name="process_elaborated_type_ref">
    <xsl:apply-templates select="type" mode="processing_elaborated_type"/>
  </xsl:template>

  <xsl:template name="process_elaborated">
    <xsl:variable
      name="nesting_depth"
      select="(string-length(compoundname)-string-length(translate(compoundname,':',''))) div 2"
    />
    <xsl:variable name="scope_full_name">
      <xsl:call-template name="nested_name_head">
        <xsl:with-param name="full_name" select="compoundname"/>
        <xsl:with-param name="depth" select="$nesting_depth"/>
      </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="base_name">
      <xsl:call-template name="nested_name_tail">
        <xsl:with-param name="full_name" select="compoundname"/>
        <xsl:with-param name="depth" select="$nesting_depth"/>
      </xsl:call-template>
    </xsl:variable>

    <xsl:choose>
      <xsl:when test="$nesting_depth = 0">
        <xsl:value-of select="$macro_prefix"/>
        <xsl:text>MIRROR_REG_GLOBAL_SCOPE_CLASS_BEGIN(</xsl:text>
          <xsl:value-of select="@kind"/>
          <xsl:text>, </xsl:text>
          <xsl:value-of select="compoundname"/>
        <xsl:text>)</xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$macro_prefix"/>
        <xsl:text>MIRROR_REG_CLASS_BEGIN(</xsl:text>
          <xsl:value-of select="@kind"/>
          <xsl:text>, </xsl:text>
          <xsl:value-of select="$scope_full_name"/>
          <xsl:text>, </xsl:text>
          <xsl:value-of select="$base_name"/>
        <xsl:text>)</xsl:text>
       </xsl:otherwise>
    </xsl:choose>
    <xsl:call-template name="newline"/>
    <!-- base-classes -->
    <xsl:if test="basecompoundref">
      <xsl:text> </xsl:text>
      <xsl:value-of select="$macro_prefix"/>
      <xsl:text>MIRROR_REG_BASE_CLASSES_BEGIN</xsl:text>
      <xsl:call-template name="newline"/>
        <xsl:for-each select="basecompoundref">
          <xsl:text>  </xsl:text>
          <xsl:value-of select="$macro_prefix"/>
          <xsl:text>MIRROR_REG_BASE_CLASS(</xsl:text>
          <xsl:choose>
            <xsl:when test="@virt = 'virtual'">
              <xsl:text>virtual</xsl:text>
            </xsl:when>
            <xsl:otherwise>
              <xsl:text>_</xsl:text>
            </xsl:otherwise>
          </xsl:choose>
          <xsl:text>, </xsl:text>
          <xsl:value-of select="@prot"/>
          <xsl:text>, </xsl:text>
          <xsl:value-of select="text()"/>
          <xsl:text>)</xsl:text>
          <xsl:if test="position() != last()">
            <xsl:text>, </xsl:text>
          </xsl:if>
          <xsl:call-template name="newline"/>
        </xsl:for-each>
      <xsl:text> </xsl:text>
      <xsl:value-of select="$macro_prefix"/>
      <xsl:text>MIRROR_REG_BASE_CLASSES_END</xsl:text>
      <xsl:call-template name="newline"/>
    </xsl:if>
    <!-- member variables -->
    <xsl:if test="sectiondef/memberdef[
                @kind = 'variable' and
                contains(definition, concat(../../compoundname,'::', name))
    ] ">
      <xsl:text> </xsl:text>
      <xsl:value-of select="$macro_prefix"/>
      <xsl:text>MIRROR_REG_CLASS_MEM_VARS_BEGIN</xsl:text>
      <xsl:call-template name="newline"/>
      <xsl:for-each select="sectiondef/memberdef[
                  @kind = 'variable' and
                  contains(definition, concat(../../compoundname,'::', name))
      ]">
        <xsl:text>  </xsl:text>
        <xsl:value-of select="$macro_prefix"/>
        <xsl:text>MIRROR_REG_CLASS_MEM_VAR(</xsl:text>
          <xsl:value-of select="@prot"/>
          <xsl:text>, </xsl:text>
          <xsl:choose>
            <xsl:when test="@static = 'yes'">
              <xsl:text>static</xsl:text>
            </xsl:when>
            <xsl:when test="@mutable = 'yes'">
              <xsl:text>mutable</xsl:text>
            </xsl:when>
            <xsl:otherwise>_</xsl:otherwise>
          </xsl:choose>
          <xsl:text>, </xsl:text>
          <xsl:call-template name="process_elaborated_type_ref"/>
          <xsl:text>, </xsl:text>
          <xsl:value-of select="name"/>
        <xsl:text>)</xsl:text>
        <xsl:call-template name="newline"/>
      </xsl:for-each>
      <xsl:text> </xsl:text>
      <xsl:value-of select="$macro_prefix"/>
      <xsl:text>MIRROR_REG_CLASS_MEM_VARS_END</xsl:text>
      <xsl:call-template name="newline"/>
    </xsl:if>
    <!-- constructors -->
    <xsl:if test="sectiondef/memberdef[
                @kind = 'function' and
                definition = concat(../../compoundname, '::', $base_name)
    ]">
      <xsl:text> </xsl:text>
      <xsl:value-of select="$macro_prefix"/>
      <xsl:text>MIRROR_REG_CONSTRUCTORS_BEGIN</xsl:text>
      <xsl:call-template name="newline"/>
      <xsl:for-each select="sectiondef/memberdef[
                  @kind = 'function' and
                  definition = concat(../../compoundname, '::', $base_name)
      ]">
        <xsl:choose>
          <xsl:when test="
                  (count(param) = 0) or
                  ((count(param) = 1) and (param/type/text() = 'void'))
          ">
            <xsl:text>  </xsl:text>
            <xsl:value-of select="$macro_prefix"/>
            <xsl:text>MIRROR_REG_DEFAULT_CONSTRUCTOR(</xsl:text>
            <xsl:value-of select="@prot"/>
            <xsl:text>)</xsl:text>
            <xsl:call-template name="newline"/>
          </xsl:when>
          <!-- TODO: this is not very clever -->
          <xsl:when test="
                  ((count(param) = 1) and contains(param/type/text(), $base_name))
          ">
            <xsl:text>  </xsl:text>
            <xsl:value-of select="$macro_prefix"/>
            <xsl:text>MIRROR_REG_COPY_CONSTRUCTOR(</xsl:text>
            <xsl:value-of select="@prot"/>
            <xsl:text>)</xsl:text>
            <xsl:call-template name="newline"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:text>  </xsl:text>
            <xsl:value-of select="$macro_prefix"/>
            <xsl:text>MIRROR_REG_CONSTRUCTOR_BEGIN(</xsl:text>
            <xsl:value-of select="@prot"/>
            <xsl:text>, </xsl:text>
            <xsl:value-of select="position()"/>
            <xsl:text>)</xsl:text>
            <xsl:call-template name="newline"/>

            <xsl:for-each select="param">
              <xsl:text>   </xsl:text>
              <xsl:value-of select="$macro_prefix"/>
              <xsl:text>MIRROR_REG_CONSTRUCTOR_PARAM(</xsl:text>
              <xsl:call-template name="process_elaborated_type_ref"/>
              <xsl:text>, </xsl:text>
              <xsl:value-of select="declname"/>
              <xsl:text>)</xsl:text>
              <xsl:call-template name="newline"/>
            </xsl:for-each>

            <xsl:text>  </xsl:text>
            <xsl:value-of select="$macro_prefix"/>
            <xsl:text>MIRROR_REG_CONSTRUCTOR_END(</xsl:text>
            <xsl:value-of select="position()"/>
            <xsl:text>)</xsl:text>
            <xsl:call-template name="newline"/>
           </xsl:otherwise>
        </xsl:choose>
      </xsl:for-each>
      <xsl:text> </xsl:text>
      <xsl:value-of select="$macro_prefix"/>
      <xsl:text>MIRROR_REG_CONSTRUCTORS_END</xsl:text>
      <xsl:call-template name="newline"/>
    </xsl:if>
     <!-- member function -->
    <xsl:if test="sectiondef/memberdef[
                @kind = 'function' and
                contains(definition, concat(type, ' ', ../../compoundname)) and
                definition != concat(../../compoundname, '::', $base_name)
    ]">
      <xsl:text> </xsl:text>
      <xsl:value-of select="$macro_prefix"/>
      <xsl:text>MIRROR_REG_MEM_FUNCTIONS_BEGIN</xsl:text>
      <xsl:call-template name="newline"/>
      <xsl:for-each select="sectiondef/memberdef[
                          @kind = 'function' and
                          contains(definition, concat(type, ' ', ../../compoundname)) and
                          definition != concat(../../compoundname, '::', $base_name)
      ]">
        <xsl:text>  </xsl:text>
        <xsl:value-of select="$macro_prefix"/>
        <xsl:text>MIRROR_REG_MEM_FUNCTION_BEGIN(</xsl:text>
        <xsl:value-of select="@prot"/>
        <xsl:text>, </xsl:text>
          <xsl:choose>
            <xsl:when test="@static = 'yes'">
              <xsl:text>static</xsl:text>
            </xsl:when>
            <!-- TODO: virtual
            <xsl:when test="@virtual = 'yes'">
              <xsl:text>virtual</xsl:text>
            </xsl:when>
             -->
            <xsl:otherwise>_</xsl:otherwise>
          </xsl:choose>
         <xsl:text>, </xsl:text>
        <xsl:call-template name="process_elaborated_type_ref"/>
        <xsl:text>, </xsl:text>
        <xsl:value-of select="name"/>
        <xsl:text>, </xsl:text>
        <xsl:value-of select="position()"/>
        <xsl:text>)</xsl:text>
        <xsl:call-template name="newline"/>

        <xsl:if test="not(
                    (count(param) = 0) or
                    ((count(param)) = 1 and (param/type/text() = 'void'))
        )">
          <xsl:for-each select="param">
            <xsl:text>   </xsl:text>
            <xsl:value-of select="$macro_prefix"/>
            <xsl:text>MIRROR_REG_MEM_FUNCTION_PARAM(</xsl:text>
            <xsl:call-template name="process_elaborated_type_ref"/>
            <xsl:text>, </xsl:text>
            <xsl:value-of select="declname"/>
            <xsl:text>)</xsl:text>
            <xsl:call-template name="newline"/>
          </xsl:for-each>
        </xsl:if>

        <xsl:text>  </xsl:text>
        <xsl:value-of select="$macro_prefix"/>
        <xsl:text>MIRROR_REG_MEM_FUNCTION_END(</xsl:text>
        <xsl:value-of select="position()"/>
        <xsl:text>)</xsl:text>
        <xsl:call-template name="newline"/>
      </xsl:for-each>
      <xsl:text> </xsl:text>
      <xsl:value-of select="$macro_prefix"/>
      <xsl:text>MIRROR_REG_MEM_FUNCTIONS_END</xsl:text>
      <xsl:call-template name="newline"/>
    </xsl:if>
    <xsl:value-of select="$macro_prefix"/>
    <xsl:text>MIRROR_REG_CLASS_END</xsl:text>
    <xsl:call-template name="newline"/>
  </xsl:template>
</xsl:stylesheet>
