<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
<!--
  - Copyright 2008-2010 Matus Chochlik. Distributed under the Boost
  - Software License, Version 1.0. (See accompanying file
  - LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
  -->

  <!-- template returning the substring before the depth-th '::' -->
  <xsl:template name="nested_name_head">
    <xsl:param name="full_name"/>
    <xsl:param name="depth"/>
    <xsl:param name="result"/>
    <xsl:param name="separator"/>
    <xsl:choose>
      <xsl:when test="$depth = 0">
        <xsl:value-of select="$result"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:call-template name="nested_name_head">
          <xsl:with-param name="full_name" select="substring-after($full_name, '::')"/>
          <xsl:with-param name="depth" select="$depth - 1"/>
          <xsl:with-param name="result" select="concat($result, $separator, substring-before($full_name, '::'))"/>
          <xsl:with-param name="separator" select="'::'"/>
        </xsl:call-template>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <!-- template returning the substring after the depth-th '::' -->
  <xsl:template name="nested_name_tail">
    <xsl:param name="full_name"/>
    <xsl:param name="depth"/>
    <xsl:choose>
      <xsl:when test="$depth = 0">
        <xsl:value-of select="$full_name"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:call-template name="nested_name_tail">
          <xsl:with-param name="full_name" select="substring-after($full_name, '::')"/>
          <xsl:with-param name="depth" select="$depth - 1"/>
        </xsl:call-template>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <!-- template returning the substring between the depth_from-th and the depth_to-th '::' -->
  <xsl:template name="nested_name_mid">
    <xsl:param name="full_name"/>
    <xsl:param name="depth_from"/>
    <xsl:param name="depth_to"/>
    <xsl:call-template name="nested_name_head">
      <xsl:with-param name="full_name">
        <xsl:call-template name="nested_name_tail">
          <xsl:with-param name="full_name" select="$full_name"/>
          <xsl:with-param name="depth" select="$depth_from"/>
        </xsl:call-template>
      </xsl:with-param>
      <xsl:with-param name="depth" select="$depth_to - $depth_from"/>
    </xsl:call-template>
  </xsl:template>

</xsl:stylesheet>
