<xsl:stylesheet version="1.0"
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:str="http://exslt.org/strings"
  xmlns:exsl="http://exslt.org/common"
  extension-element-prefixes="str exsl"
>
<!--
  - Copyright 2008-2010 Matus Chochlik. Distributed under the Boost
  - Software License, Version 1.0. (See accompanying file
  - LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
  -->

  <!-- processes all items -->
  <xsl:template name="process_items">
    <xsl:element name="items">
      <xsl:for-each select="compound[@kind='namespace']">
        <xsl:call-template name="process_item"/>
      </xsl:for-each>
      <xsl:for-each select="compound[@kind='class' or @kind='struct' or @kind='union']">
        <xsl:call-template name="process_item"/>
      </xsl:for-each>
    </xsl:element>
  </xsl:template>

  <xsl:template name="expand_reference">
    <xsl:param name="str"/>
    <xsl:param name="var"/>
    <xsl:param name="val"/>
    <xsl:variable name="ref" select="concat('$(',$var,')')"/>
    <xsl:variable name="repref" select="concat('$(',$var,':')"/>
    <xsl:choose>
      <xsl:when test="contains($str, $ref)">
        <xsl:value-of select="substring-before($str, $ref)"/>
        <xsl:value-of select="$val"/>
        <xsl:value-of select="substring-after($str, $ref)"/>
      </xsl:when>
      <xsl:when test="contains($str, $repref)">
        <xsl:value-of select="substring-before($str, $repref)"/>
        <xsl:value-of select="str:replace($val, '::', substring-before(substring-after($str, $repref), ')'))"/>
        <xsl:value-of select="substring-after(substring-after($str, $repref), ')')"/>
      </xsl:when>
      <xsl:otherwise><xsl:value-of select="$str"/></xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template name="expand_references">
    <xsl:param name="str"/>
    <xsl:param name="item_scope"/>
    <xsl:param name="item_name"/>
    <xsl:call-template name="expand_reference">
      <xsl:with-param name="str">
        <xsl:call-template name="expand_reference">
          <xsl:with-param name="str">
            <xsl:call-template name="expand_reference">
              <xsl:with-param name="str" select="$str"/>
              <xsl:with-param name="var" select="'name'"/>
              <xsl:with-param name="val" select="$item_name"/>
            </xsl:call-template>
          </xsl:with-param>
          <xsl:with-param name="var" select="'scope'"/>
          <xsl:with-param name="val" select="$item_scope"/>
        </xsl:call-template>
      </xsl:with-param>
      <xsl:with-param name="var" select="'kind'"/>
      <xsl:with-param name="val" select="@kind"/>
    </xsl:call-template>
  </xsl:template>

  <xsl:template name="compose_filepath">
    <xsl:param name="root"/>
    <xsl:param name="path"/>
    <xsl:param name="file"/>
    <xsl:param name="suffix"/>

    <xsl:if test="$root!='' and $root!='.'"><xsl:value-of select="$root"/>/</xsl:if>
    <xsl:if test="$path!='' and $path!='.'"><xsl:value-of select="$path"/>/</xsl:if>
    <xsl:value-of select="$file"/>
    <xsl:if test="$suffix!=''">.<xsl:value-of select="$suffix"/></xsl:if>
  </xsl:template>

  <!-- processes a single item like namespace, class, struct, etc. -->
  <xsl:template name="process_item">
    <!-- get the scope name and the base name of the item -->
    <xsl:variable
      name="nesting_depth"
      select="(string-length(name)-string-length(translate(name,':',''))) div 2"
    />
    <xsl:variable name="item_kind" select="@kind"/>
    <xsl:variable name="item_scope">
      <xsl:call-template name="nested_name_head">
        <xsl:with-param name="full_name" select="name"/>
        <xsl:with-param name="depth" select="$nesting_depth"/>
      </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="item_name">
      <xsl:call-template name="nested_name_tail">
        <xsl:with-param name="full_name" select="name"/>
        <xsl:with-param name="depth" select="$nesting_depth"/>
      </xsl:call-template>
    </xsl:variable>

    <xsl:variable
      name="item_def"
      select="document(concat($work_dir, '/xml/', @refid, '.xml'))/doxygen/compounddef"
    />
    <xsl:variable name="full_path">
      <xsl:value-of select="exsl:node-set($item_def)/location/@file"/>
    </xsl:variable>
    <xsl:variable name="item_config"
      select="document(concat($work_dir,'/config.xml'))/doxyreg/input[
        (@full_path = $full_path)  and
        (@kind = '%' or @kind = $item_kind) and
        (@scope = $item_scope or (
          contains(@scope, '%') and $item_scope != '' and
          starts-with($item_scope, substring-before(@scope, '%')) and (
            substring-after(@scope, '%') = '' or
            substring-after($item_scope, substring-after(@scope, '%')) = ''
          )
        )) and
        (@name = '%' or @name = $item_name)
      ]"
    />
    <xsl:choose>
      <xsl:when test="count(exsl:node-set($item_config)) = 1">
        <xsl:element name="item">
          <xsl:attribute name="input_include">
            <xsl:call-template name="compose_filepath">
              <xsl:with-param name="path" select="exsl:node-set($item_config)/@path"/>
              <xsl:with-param name="file" select="exsl:node-set($item_config)/@file"/>
              <xsl:with-param name="suffix" select="exsl:node-set($item_config)/@suffix"/>
            </xsl:call-template>
          </xsl:attribute>
          <xsl:attribute name="output_rel_path">
            <xsl:call-template name="compose_filepath">
              <xsl:with-param name="root">
                <xsl:call-template name="expand_references">
                  <xsl:with-param name="str" select="exsl:node-set($item_config)/output/@root"/>
                  <xsl:with-param name="item_scope" select="$item_scope"/>
                  <xsl:with-param name="item_name" select="$item_name"/>
                </xsl:call-template>
              </xsl:with-param>
              <xsl:with-param name="path">
                <xsl:call-template name="expand_references">
                  <xsl:with-param name="str" select="exsl:node-set($item_config)/output/@path"/>
                  <xsl:with-param name="item_scope" select="$item_scope"/>
                  <xsl:with-param name="item_name" select="$item_name"/>
                </xsl:call-template>
              </xsl:with-param>
              <xsl:with-param name="file">
                <xsl:call-template name="expand_references">
                  <xsl:with-param name="str" select="exsl:node-set($item_config)/output/@file"/>
                  <xsl:with-param name="item_scope" select="$item_scope"/>
                  <xsl:with-param name="item_name" select="$item_name"/>
                </xsl:call-template>
              </xsl:with-param>
              <xsl:with-param name="suffix">
                <xsl:call-template name="expand_references">
                  <xsl:with-param name="str" select="exsl:node-set($item_config)/output/@suffix"/>
                  <xsl:with-param name="item_scope" select="$item_scope"/>
                  <xsl:with-param name="item_name" select="$item_name"/>
                </xsl:call-template>
              </xsl:with-param>
            </xsl:call-template>
          </xsl:attribute>
          <xsl:call-template name="newline"/>
          <xsl:for-each select="exsl:node-set($item_def)">
            <xsl:choose>
              <xsl:when test="$item_kind='namespace'">
                <xsl:call-template name="process_namespace"/>
              </xsl:when>
              <xsl:when test="$item_kind='class' or $item_kind='struct' or $item_kind='union'">
                <xsl:call-template name="process_elaborated"/>
              </xsl:when>
            </xsl:choose>
          </xsl:for-each>
        </xsl:element>
      </xsl:when>
      <xsl:when test="count(exsl:node-set($item_config)) &gt; 1">
        <xsl:message terminate="no">Ambiguous transformation rule. Skipping.</xsl:message>
      </xsl:when>
    </xsl:choose>
  </xsl:template>


</xsl:stylesheet>
